/**************************************************************************

Copyright (c) 2016, Intel Corporation

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of Intel Corporation nor the names of its contributors
      may be used to endorse or promote products derived from this software
      without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

***************************************************************************/

#ifndef _ICE_FDIR_H_
#define _ICE_FDIR_H_

#include "ice_common.h"

#ifndef ICE_FDIR_IP_PROTOCOLS
#define ICE_FDIR_IP_PROTOCOLS
#define ICE_IP_PROTO_TCP		6
#define ICE_IP_PROTO_UDP		17
#define ICE_IP_PROTO_SCTP		132
#define ICE_IP_PROTO_IP			0
#define ICE_IP_PROTO_ESP		50
#endif /* ICE_FDIR_IP_PROTOCOLS */

#define ICE_FDIR_TUN_PKT_OFF		50
#define ICE_FDIR_MAX_RAW_PKT_SIZE	(512 + ICE_FDIR_TUN_PKT_OFF)
#ifndef NO_UNUSED_PACKAGE_CODE
#define ICE_FDIR_BUF_FULL_MARGIN	10
#endif /* !NO_UNUSED_PACKAGE_CODE */
#ifdef ATR_SUPPORT
#define ICE_FDIR_BUF_HEAD_ROOM		32
#define ICE_FDIR_BUF_HEAD_ROOM_FOR_ATR (ICE_FDIR_BUF_HEAD_ROOM * 4)
#define ICE_DFLT_ATR_SAMPLE_RATE	20
#endif /* ATR_SUPPORT */

/* macros for offsets into packets for flow director programming */
#define ICE_IPV4_SRC_ADDR_OFFSET	26
#define ICE_IPV4_DST_ADDR_OFFSET	30
#define ICE_IPV4_TCP_SRC_PORT_OFFSET	34
#define ICE_IPV4_TCP_DST_PORT_OFFSET	36
#define ICE_IPV4_UDP_SRC_PORT_OFFSET	34
#define ICE_IPV4_UDP_DST_PORT_OFFSET	36
#define ICE_IPV4_SCTP_SRC_PORT_OFFSET	34
#define ICE_IPV4_SCTP_DST_PORT_OFFSET	36
#define ICE_IPV4_PROTO_OFFSET		23
#define ICE_IPV6_SRC_ADDR_OFFSET	22
#define ICE_IPV6_DST_ADDR_OFFSET	38
#define ICE_IPV6_TCP_SRC_PORT_OFFSET	54
#define ICE_IPV6_TCP_DST_PORT_OFFSET	56
#define ICE_IPV6_UDP_SRC_PORT_OFFSET	54
#define ICE_IPV6_UDP_DST_PORT_OFFSET	56
#define ICE_IPV6_SCTP_SRC_PORT_OFFSET	54
#define ICE_IPV6_SCTP_DST_PORT_OFFSET	56

#if defined(DPDK_SUPPORT) || defined(ADV_AVF_SUPPORT)
#define ICE_MAC_ETHTYPE_OFFSET		12
#define ICE_IPV4_TOS_OFFSET		15
#define ICE_IPV4_TTL_OFFSET		22
#define ICE_IPV6_TC_OFFSET		14
#define ICE_IPV6_HLIM_OFFSET		21
#define ICE_IPV6_PROTO_OFFSET		20
#define ICE_IPV4_GTPU_TEID_OFFSET	46
#define ICE_IPV4_GTPU_QFI_OFFSET	56
#define ICE_IPV6_GTPU_TEID_OFFSET	66
#define ICE_IPV6_GTPU_QFI_OFFSET	76
#define ICE_IPV4_L2TPV3_SESS_ID_OFFSET	34
#define ICE_IPV6_L2TPV3_SESS_ID_OFFSET	54
#define ICE_IPV4_ESP_SPI_OFFSET		34
#define ICE_IPV6_ESP_SPI_OFFSET		54
#define ICE_IPV4_AH_SPI_OFFSET		38
#define ICE_IPV6_AH_SPI_OFFSET		58
#define ICE_IPV4_NAT_T_ESP_SPI_OFFSET	42
#define ICE_IPV6_NAT_T_ESP_SPI_OFFSET	62
#define ICE_IPV4_VXLAN_VNI_OFFSET	45
#define ICE_ECPRI_TP0_PC_ID_OFFSET	18
#define ICE_IPV4_UDP_ECPRI_TP0_PC_ID_OFFSET			46
#endif /* DPDK_SUPPORT || ADV_AVF_SUPPORT */

#if defined(FDIR_SUPPORT) && defined(ADV_AVF_SUPPORT)
#ifndef SIMICS_SUPPORT
#define ICE_FDIR_MAX_FLTRS		16384
#endif /* SIMICS_SUPPORT */
#endif /* FDIR_SUPPORT && ADV_AVF_SUPPORT */

/* IP v4 has 2 flag bits that enable fragment processing: DF and MF. DF
 * requests that the packet not be fragmented. MF indicates that a packet has
 * been fragmented.
 */
#define ICE_FDIR_IPV4_PKT_FLAG_DF		0x20
#ifndef NO_UNUSED_PACKAGE_CODE
#define ICE_FDIR_IPV4_PKT_FLAG_MF		0x40
#endif /* NO_UNUSED_PACKAGE_CODE */

enum ice_fltr_prgm_desc_dest {
	ICE_FLTR_PRGM_DESC_DEST_DROP_PKT,
	ICE_FLTR_PRGM_DESC_DEST_DIRECT_PKT_QINDEX,
	ICE_FLTR_PRGM_DESC_DEST_DIRECT_PKT_QGROUP,
	ICE_FLTR_PRGM_DESC_DEST_DIRECT_PKT_OTHER,
};

enum ice_fltr_prgm_desc_fd_status {
	ICE_FLTR_PRGM_DESC_FD_STATUS_NONE,
	ICE_FLTR_PRGM_DESC_FD_STATUS_FD_ID,
	ICE_FLTR_PRGM_DESC_FD_STATUS_FD_ID_4FLEX_BYTES,
	ICE_FLTR_PRGM_DESC_FD_STATUS_8FLEX_BYTES,
};

/* Flow Director (FD) Filter Programming descriptor */
struct ice_fd_fltr_desc_ctx {
	u32 fdid;
	u16 qindex;
	u16 cnt_index;
	u16 fd_vsi;
	u16 flex_val;
	u8 comp_q;
	u8 comp_report;
	u8 fd_space;
	u8 cnt_ena;
	u8 evict_ena;
	u8 toq;
	u8 toq_prio;
	u8 dpu_recipe;
	u8 drop;
	u8 flex_prio;
	u8 flex_mdid;
	u8 dtype;
	u8 pcmd;
	u8 desc_prof_prio;
	u8 desc_prof;
	u8 swap;
	u8 fdid_prio;
	u8 fdid_mdid;
};

#define ICE_FLTR_PRGM_FLEX_WORD_SIZE	sizeof(__be16)

struct ice_rx_flow_userdef {
	u16 flex_word;
	u16 flex_offset;
	u16 flex_fltr;
};

struct ice_fdir_v4 {
	__be32 dst_ip;
	__be32 src_ip;
	__be16 dst_port;
	__be16 src_port;
	__be32 l4_header;
	__be32 sec_parm_idx;	/* security parameter index */
	u8 tos;
	u8 ip_ver;
	u8 proto;
#if defined(DPDK_SUPPORT) || defined(ADV_AVF_SUPPORT)
	u8 ttl;
#endif /* DPDK_SUPPORT || ADV_AVF_SUPPORT */
};

#define ICE_IPV6_ADDR_LEN_AS_U32		4

struct ice_fdir_v6 {
	__be32 dst_ip[ICE_IPV6_ADDR_LEN_AS_U32];
	__be32 src_ip[ICE_IPV6_ADDR_LEN_AS_U32];
	__be16 dst_port;
	__be16 src_port;
	__be32 l4_header; /* next header */
	__be32 sec_parm_idx; /* security parameter index */
	u8 tc;
	u8 proto;
#if defined(DPDK_SUPPORT) || defined(ADV_AVF_SUPPORT)
	u8 hlim;
#endif /* DPDK_SUPPORT || ADV_AVF_SUPPORT */
};

#if defined(DPDK_SUPPORT) || defined(ADV_AVF_SUPPORT)
struct ice_fdir_udp_gtp {
	u8 flags;
	u8 msg_type;
	__be16 rsrvd_len;
	__be32 teid;
	__be16 rsrvd_seq_nbr;
	u8 rsrvd_n_pdu_nbr;
	u8 rsrvd_next_ext_type;
	u8 rsvrd_ext_len;
	u8	pdu_type:4,
		spare:4;
	u8	ppp:1,
		rqi:1,
		qfi:6;
	u32 rsvrd;
	u8 next_ext;
};

struct ice_fdir_l2tpv3 {
	__be32 session_id;
};

struct ice_fdir_udp_vxlan {
	__be32 vni; /* 8 bits reserved, always be zero */
};

struct ice_fdir_ecpri {
	__be16 pc_id;
};
#endif /* DPDK_SUPPORT || ADV_AVF_SUPPORT */

struct ice_fdir_extra {
	u8 dst_mac[ETH_ALEN];	/* dest MAC address */
#if defined(DPDK_SUPPORT) || defined(ADV_AVF_SUPPORT)
	u8 src_mac[ETH_ALEN];	/* src MAC address */
	__be16 ether_type;      /* for NON_IP_L2 */
#endif /* DPDK_SUPPORT || ADV_AVF_SUPPORT */
	u32 usr_def[2];		/* user data */
	__be16 vlan_type;	/* VLAN ethertype */
	__be16 vlan_tag;	/* VLAN tag info */
};

struct ice_fdir_fltr {
	struct LIST_ENTRY_TYPE fltr_node;
	enum ice_fltr_ptype flow_type;

	union {
		struct ice_fdir_v4 v4;
		struct ice_fdir_v6 v6;
	} ip, mask;

#if defined(DPDK_SUPPORT) || defined(ADV_AVF_SUPPORT)
	/* for tunnel outer part */
	union {
		struct ice_fdir_v4 v4;
		struct ice_fdir_v6 v6;
	} ip_outer, mask_outer;

	struct ice_fdir_extra ext_data_outer;
	struct ice_fdir_extra ext_mask_outer;

	struct ice_fdir_udp_vxlan vxlan_data;
	struct ice_fdir_udp_vxlan vxlan_mask;

	struct ice_fdir_udp_gtp gtpu_data;
	struct ice_fdir_udp_gtp gtpu_mask;

	struct ice_fdir_l2tpv3 l2tpv3_data;
	struct ice_fdir_l2tpv3 l2tpv3_mask;

	struct ice_fdir_ecpri ecpri_data;
	struct ice_fdir_ecpri ecpri_mask;
#endif /* DPDK_SUPPORT || ADV_AVF_SUPPORT */

	struct ice_fdir_extra ext_data;
	struct ice_fdir_extra ext_mask;

	/* flex byte filter data */
	__be16 flex_word;
#if defined(DPDK_SUPPORT) || defined(ADV_AVF_SUPPORT)
	/* queue region size (=2^q_region) */
	u8 q_region;
#endif /* DPDK_SUPPORT || ADV_AVF_SUPPORT */
	u16 flex_offset;
	u16 flex_fltr;

	/* filter control */
	u16 q_index;
#ifdef ADQ_SUPPORT
	u16 orig_q_index;
#endif /* ADQ_SUPPORT */
	u16 dest_vsi;
	u8 dest_ctl;
	u8 cnt_ena;
	u8 fltr_status;
	u16 cnt_index;
	u32 fltr_id;
	u8 fdid_prio;
	u8 comp_report;
#ifndef NO_ACL_SUPPORT
	/* Set to true for an ACL filter */
	bool acl_fltr;
#endif /* !NO_ACL_SUPPORT */
};

/* Dummy packet filter definition structure */
struct ice_fdir_base_pkt {
	enum ice_fltr_ptype flow;
	u16 pkt_len;
	const u8 *pkt;
	u16 tun_pkt_len;
	const u8 *tun_pkt;
};

#ifndef ICE_TDD
enum ice_status ice_alloc_fd_res_cntr(struct ice_hw *hw, u16 *cntr_id);
enum ice_status ice_free_fd_res_cntr(struct ice_hw *hw, u16 cntr_id);
#ifdef ADQ_SUPPORT
void
ice_set_fd_desc_val(struct ice_fd_fltr_desc_ctx *fd_fltr_ctx,
		    struct ice_fltr_desc *fdir_desc);
void ice_set_dflt_val_fd_desc(struct ice_fd_fltr_desc_ctx *fd_fltr_ctx);
#endif /* ADQ_SUPPORT */
enum ice_status
ice_alloc_fd_guar_item(struct ice_hw *hw, u16 *cntr_id, u16 num_fltr);
#ifndef NO_UNUSED_PACKAGE_CODE
enum ice_status
ice_free_fd_guar_item(struct ice_hw *hw, u16 cntr_id, u16 num_fltr);
#endif /* !NO_UNUSED_PACKAGE_CODE */
enum ice_status
ice_alloc_fd_shrd_item(struct ice_hw *hw, u16 *cntr_id, u16 num_fltr);
#ifndef NO_UNUSED_PACKAGE_CODE
enum ice_status
ice_free_fd_shrd_item(struct ice_hw *hw, u16 cntr_id, u16 num_fltr);
#endif /* !NO_UNUSED_PACKAGE_CODE */
#ifdef ADQ_SUPPORT
enum ice_status ice_clear_vsi_fd_table(struct ice_hw *hw, u16 vsi_num);
#endif /* ADQ_SUPPORT */
#ifndef NO_UNUSED_PACKAGE_CODE
enum ice_status ice_clear_pf_fd_table(struct ice_hw *hw);
#endif /* !NO_UNUSED_PACKAGE_CODE */
void
ice_fdir_get_prgm_desc(struct ice_hw *hw, struct ice_fdir_fltr *input,
		       struct ice_fltr_desc *fdesc, bool add);
enum ice_status
ice_fdir_get_gen_prgm_pkt(struct ice_hw *hw, struct ice_fdir_fltr *input,
			  u8 *pkt, bool frag, bool tun);
#ifndef NO_UNUSED_PACKAGE_CODE
enum ice_status
ice_fdir_get_prgm_pkt(struct ice_fdir_fltr *input, u8 *pkt, bool frag);
#endif /* !NO_UNUSED_PACKAGE_CODE */
int ice_get_fdir_cnt_all(struct ice_hw *hw);
bool ice_fdir_is_dup_fltr(struct ice_hw *hw, struct ice_fdir_fltr *input);
bool ice_fdir_has_frag(enum ice_fltr_ptype flow);
struct ice_fdir_fltr *
ice_fdir_find_fltr_by_idx(struct ice_hw *hw, u32 fltr_idx);
void
ice_fdir_update_cntrs(struct ice_hw *hw, enum ice_fltr_ptype flow,
		      bool acl_fltr, bool add);
void ice_fdir_list_add_fltr(struct ice_hw *hw, struct ice_fdir_fltr *input);
#endif /* ICE_TDD */
#endif /* _ICE_FDIR_H_ */
